package net.krakatoaapi.builder;

import java.util.Arrays;
import org.bukkit.Color;
import org.bukkit.Material;
import org.bukkit.enchantments.Enchantment;
import org.bukkit.inventory.ItemFlag;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.inventory.meta.LeatherArmorMeta;
import org.bukkit.inventory.meta.SkullMeta;

public class ItemBuilder {

  private final ItemStack itemStack;

  public ItemBuilder(Material mat, int amount, short type) {
    this.itemStack = new ItemStack(mat, amount, type);
  }

  public ItemBuilder(Material mat) {
    this.itemStack = new ItemStack(mat);
  }

  public ItemBuilder(Material material, int amount) {
    this.itemStack = new ItemStack(material, amount);
  }

  public ItemBuilder setDisplayname(String displayname) {
    ItemMeta im = this.itemStack.getItemMeta();
    assert im != null;
    im.setDisplayName(displayname);
    this.itemStack.setItemMeta(im);
    return this;
  }

  public ItemBuilder addEnchantment(Enchantment enchantment, Integer level) {
    ItemMeta im = this.itemStack.getItemMeta();
    assert im != null;
    im.addEnchant(enchantment, level, true);
    this.itemStack.setItemMeta(im);
    return this;
  }

  public ItemBuilder addUnsafeEnchantment(Enchantment enchantment, Integer enchantmentLevel) {
    ItemMeta itemMeta = this.itemStack.getItemMeta();
    itemStack.addUnsafeEnchantment(enchantment, enchantmentLevel);
    this.itemStack.setItemMeta(itemMeta);
    return this;
  }

  public ItemBuilder setColor(int red, int green, int blue) {
    LeatherArmorMeta leatherArmorMeta = (LeatherArmorMeta) this.itemStack.getItemMeta();
    assert leatherArmorMeta != null;
    leatherArmorMeta.setColor(Color.fromBGR(red, green, blue));
    this.itemStack.setItemMeta(leatherArmorMeta);
    return this;
  }

  public ItemBuilder setColor(Color color) {
    LeatherArmorMeta leatherArmorMeta = (LeatherArmorMeta) this.itemStack.getItemMeta();
    assert leatherArmorMeta != null;
    leatherArmorMeta.setColor(color);
    this.itemStack.setItemMeta(leatherArmorMeta);
    return this;
  }

  public ItemBuilder setLore(String... lore) {
    ItemMeta im = this.itemStack.getItemMeta();
    assert im != null;
    im.setLore(Arrays.asList(lore));
    itemStack.setItemMeta(im);
    return this;
  }

  public ItemBuilder addItemFlag(ItemFlag itemFlag) {
    ItemMeta im = this.itemStack.getItemMeta();
    assert im != null;
    im.addItemFlags(itemFlag);
    itemStack.setItemMeta(im);
    return this;
  }

  public ItemBuilder setUnbreakable(boolean unbreakable) {
    ItemMeta itemMeta = this.itemStack.getItemMeta();
    assert itemMeta != null;
    itemMeta.setUnbreakable(unbreakable);
    return this;
  }

  public ItemBuilder setSkullOwner(String skullOwner) {
    SkullMeta skullMeta = (SkullMeta) this.itemStack.getItemMeta();
    assert skullMeta != null;
    skullMeta.setOwner(skullOwner);
    this.itemStack.setItemMeta(skullMeta);
    return this;
  }

  public ItemBuilder hideEnchantment() {
    ItemMeta itemMeta = this.itemStack.getItemMeta();
    assert itemMeta != null;
    itemMeta.addItemFlags(ItemFlag.HIDE_ENCHANTS);
    this.itemStack.setItemMeta(itemMeta);
    return this;
  }

  public ItemBuilder hideAttributes() {
    ItemMeta itemMeta = this.itemStack.getItemMeta();
    assert itemMeta != null;
    itemMeta.addItemFlags(ItemFlag.HIDE_ATTRIBUTES);
    this.itemStack.setItemMeta(itemMeta);
    return this;
  }

  public ItemBuilder setGlowing() {
    ItemMeta itemMeta = this.itemStack.getItemMeta();
    assert itemMeta != null;
    itemMeta.addItemFlags(ItemFlag.HIDE_ENCHANTS);
    itemMeta.addEnchant(Enchantment.DURABILITY, 1, false);
    this.itemStack.setItemMeta(itemMeta);
    return this;
  }

  public ItemStack build() {
    return this.itemStack;
  }
}
