package net.krakatoaapi.config;

import java.io.File;
import java.io.IOException;
import java.util.List;
import lombok.Getter;
import org.bukkit.configuration.InvalidConfigurationException;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;

@Getter
public class ConfigHandler {

  private final String defaultConfigPath = "/home/kra-minecraft/kraConfigs";

  private String webSocketAccessKey;
  private String webSocketProtocol;
  private String webSocketHost;
  private int webSocketPort;
  private List<Long> webSocketAckTimeouts;

  private String webVoiceAddress;

  private String mongodbHost;
  private int mongodbPort;
  private String mongodbDatabase;
  private String mongodbUsername;
  private String mongodbPassword;

  private String redisHost;
  private int redisPort;

  public void load() throws IOException, InvalidConfigurationException {
    File file = new File(defaultConfigPath, "kraSettings.yaml");

    FileConfiguration fileConfiguration = new YamlConfiguration();

    fileConfiguration.load(file);

    webSocketAccessKey = fileConfiguration.getString("web_socket.access_key");
    webSocketProtocol = fileConfiguration.getString("web_socket.protocol");
    webSocketHost = fileConfiguration.getString("web_socket.host");
    webSocketPort = fileConfiguration.getInt("web_socket.port");
    webSocketAckTimeouts = fileConfiguration.getLongList("web_socket.ack_timeouts");

    webVoiceAddress = fileConfiguration.getString("web.voice_address");

    mongodbHost = fileConfiguration.getString("mongodb.host");
    mongodbPort = fileConfiguration.getInt("mongodb.port");
    mongodbDatabase = fileConfiguration.getString("mongodb.database");
    mongodbUsername = fileConfiguration.getString("mongodb.username");
    mongodbPassword = fileConfiguration.getString("mongodb.password");

    redisHost = fileConfiguration.getString("redis.host");
    redisPort = fileConfiguration.getInt("redis.port");
  }
}
